DEFAULT_MESSAGE_FIELDS = [
    "bccRecipients",
    "body",
    "ccRecipients",
    "conversationId",
    "conversationIndex",
    "flag",
    "from",
    "hasAttachments",
    "importance",
    "isDraft",
    "isRead",
    "receivedDateTime",
    "replyTo",
    "subject",
    "toRecipients",
    "webLink",
]
