COUNTRY_CODES = {
    "af": "Afghanistan",
    "al": "Albania",
    "dz": "Algeria",
    "as": "American Samoa",
    "ad": "Andorra",
    "ao": "Angola",
    "ai": "Anguilla",
    "aq": "Antarctica",
    "ag": "Antigua and Barbuda",
    "ar": "Argentina",
    "am": "Armenia",
    "aw": "Aruba",
    "au": "Australia",
    "at": "Austria",
    "az": "Azerbaijan",
    "bs": "Bahamas",
    "bh": "Bahrain",
    "bd": "Bangladesh",
    "bb": "Barbados",
    "by": "Belarus",
    "be": "Belgium",
    "bz": "Belize",
    "bj": "Benin",
    "bm": "Bermuda",
    "bt": "Bhutan",
    "bo": "Bolivia",
    "ba": "Bosnia and Herzegovina",
    "bw": "Botswana",
    "bv": "Bouvet Island",
    "br": "Brazil",
    "io": "British Indian Ocean Territory",
    "bn": "Brunei Darussalam",
    "bg": "Bulgaria",
    "bf": "Burkina Faso",
    "bi": "Burundi",
    "kh": "Cambodia",
    "cm": "Cameroon",
    "ca": "Canada",
    "cv": "Cape Verde",
    "ky": "Cayman Islands",
    "cf": "Central African Republic",
    "td": "Chad",
    "cl": "Chile",
    "cn": "China",
    "cx": "Christmas Island",
    "cc": "Cocos (Keeling) Islands",
    "co": "Colombia",
    "km": "Comoros",
    "cg": "Congo",
    "cd": "Congo, the Democratic Republic of the",
    "ck": "Cook Islands",
    "cr": "Costa Rica",
    "ci": "Cote D'ivoire",
    "hr": "Croatia",
    "cu": "Cuba",
    "cy": "Cyprus",
    "cz": "Czech Republic",
    "dk": "Denmark",
    "dj": "Djibouti",
    "dm": "Dominica",
    "do": "Dominican Republic",
    "ec": "Ecuador",
    "eg": "Egypt",
    "sv": "El Salvador",
    "gq": "Equatorial Guinea",
    "er": "Eritrea",
    "ee": "Estonia",
    "et": "Ethiopia",
    "fk": "Falkland Islands (Malvinas)",
    "fo": "Faroe Islands",
    "fj": "Fiji",
    "fi": "Finland",
    "fr": "France",
    "gf": "French Guiana",
    "pf": "French Polynesia",
    "tf": "French Southern Territories",
    "ga": "Gabon",
    "gm": "Gambia",
    "ge": "Georgia",
    "de": "Germany",
    "gh": "Ghana",
    "gi": "Gibraltar",
    "gr": "Greece",
    "gl": "Greenland",
    "gd": "Grenada",
    "gp": "Guadeloupe",
    "gu": "Guam",
    "gt": "Guatemala",
    "gg": "Guernsey",
    "gn": "Guinea",
    "gw": "Guinea-Bissau",
    "gy": "Guyana",
    "ht": "Haiti",
    "hm": "Heard Island and Mcdonald Islands",
    "va": "Holy See (Vatican City State)",
    "hn": "Honduras",
    "hk": "Hong Kong",
    "hu": "Hungary",
    "is": "Iceland",
    "in": "India",
    "id": "Indonesia",
    "ir": "Iran, Islamic Republic of",
    "iq": "Iraq",
    "ie": "Ireland",
    "im": "Isle of Man",
    "il": "Israel",
    "it": "Italy",
    "je": "Jersey",
    "jm": "Jamaica",
    "jp": "Japan",
    "jo": "Jordan",
    "kz": "Kazakhstan",
    "ke": "Kenya",
    "ki": "Kiribati",
    "kp": "Korea, Democratic People's Republic of",
    "kr": "Korea, Republic of",
    "kw": "Kuwait",
    "kg": "Kyrgyzstan",
    "la": "Lao People's Democratic Republic",
    "lv": "Latvia",
    "lb": "Lebanon",
    "ls": "Lesotho",
    "lr": "Liberia",
    "ly": "Libyan Arab Jamahiriya",
    "li": "Liechtenstein",
    "lt": "Lithuania",
    "lu": "Luxembourg",
    "mo": "Macao",
    "mk": "Macedonia, the Former Yugosalv Republic of",
    "mg": "Madagascar",
    "mw": "Malawi",
    "my": "Malaysia",
    "mv": "Maldives",
    "ml": "Mali",
    "mt": "Malta",
    "mh": "Marshall Islands",
    "mq": "Martinique",
    "mr": "Mauritania",
    "mu": "Mauritius",
    "yt": "Mayotte",
    "mx": "Mexico",
    "fm": "Micronesia, Federated States of",
    "md": "Moldova, Republic of",
    "mc": "Monaco",
    "mn": "Mongolia",
    "me": "Montenegro",
    "ms": "Montserrat",
    "ma": "Morocco",
    "mz": "Mozambique",
    "mm": "Myanmar",
    "na": "Namibia",
    "nr": "Nauru",
    "np": "Nepal",
    "nl": "Netherlands",
    "an": "Netherlands Antilles",
    "nc": "New Caledonia",
    "nz": "New Zealand",
    "ni": "Nicaragua",
    "ne": "Niger",
    "ng": "Nigeria",
    "nu": "Niue",
    "nf": "Norfolk Island",
    "mp": "Northern Mariana Islands",
    "no": "Norway",
    "om": "Oman",
    "pk": "Pakistan",
    "pw": "Palau",
    "ps": "Palestinian Territory, Occupied",
    "pa": "Panama",
    "pg": "Papua New Guinea",
    "py": "Paraguay",
    "pe": "Peru",
    "ph": "Philippines",
    "pn": "Pitcairn",
    "pl": "Poland",
    "pt": "Portugal",
    "pr": "Puerto Rico",
    "qa": "Qatar",
    "re": "Reunion",
    "ro": "Romania",
    "ru": "Russian Federation",
    "rw": "Rwanda",
    "sh": "Saint Helena",
    "kn": "Saint Kitts and Nevis",
    "lc": "Saint Lucia",
    "pm": "Saint Pierre and Miquelon",
    "vc": "Saint Vincent and the Grenadines",
    "ws": "Samoa",
    "sm": "San Marino",
    "st": "Sao Tome and Principe",
    "sa": "Saudi Arabia",
    "sn": "Senegal",
    "rs": "Serbia",
    "sc": "Seychelles",
    "sl": "Sierra Leone",
    "sg": "Singapore",
    "sk": "Slovakia",
    "si": "Slovenia",
    "sb": "Solomon Islands",
    "so": "Somalia",
    "za": "South Africa",
    "gs": "South Georgia and the South Sandwich Islands",
    "es": "Spain",
    "lk": "Sri Lanka",
    "sd": "Sudan",
    "sr": "Suriname",
    "sj": "Svalbard and Jan Mayen",
    "sz": "Swaziland",
    "se": "Sweden",
    "ch": "Switzerland",
    "sy": "Syrian Arab Republic",
    "tw": "Taiwan, Province of China",
    "tj": "Tajikistan",
    "tz": "Tanzania, United Republic of",
    "th": "Thailand",
    "tl": "Timor-Leste",
    "tg": "Togo",
    "tk": "Tokelau",
    "to": "Tonga",
    "tt": "Trinidad and Tobago",
    "tn": "Tunisia",
    "tr": "Turkiye",
    "tm": "Turkmenistan",
    "tc": "Turks and Caicos Islands",
    "tv": "Tuvalu",
    "ug": "Uganda",
    "ua": "Ukraine",
    "ae": "United Arab Emirates",
    "uk": "United Kingdom",
    "gb": "United Kingdom",
    "us": "United States",
    "um": "United States Minor Outlying Islands",
    "uy": "Uruguay",
    "uz": "Uzbekistan",
    "vu": "Vanuatu",
    "ve": "Venezuela",
    "vn": "Viet Nam",
    "vg": "Virgin Islands, British",
    "vi": "Virgin Islands, U.S.",
    "wf": "Wallis and Futuna",
    "eh": "Western Sahara",
    "ye": "Yemen",
    "zm": "Zambia",
    "zw": "Zimbabwe",
}


LANGUAGE_CODES = {
    "ar": "Arabic",
    "bn": "Bengali",
    "da": "Danish",
    "de": "German",
    "el": "Greek",
    "en": "English",
    "es": "Spanish",
    "fi": "Finnish",
    "fr": "French",
    "hi": "Hindi",
    "hu": "Hungarian",
    "id": "Indonesian",
    "it": "Italian",
    "ja": "Japanese",
    "ko": "Korean",
    "nl": "Dutch",
    "ms": "Malay",
    "no": "Norwegian",
    "pcm": "Nigerian Pidgin",
    "pl": "Polish",
    "pt": "Portuguese",
    "pt-br": "Portuguese (Brazil)",
    "pt-pt": "Portuguese (Portugal)",
    "ru": "Russian",
    "sv": "Swedish",
    "tl": "Filipino",
    "tr": "Turkish",
    "uk": "Ukrainian",
    "zh": "Chinese",
    "zh-cn": "Chinese (Simplified)",
    "zh-tw": "Chinese (Traditional)",
}


GOOGLE_DOMAIN_BY_COUNTRY_CODE = {
    "ad": "google.ad",
    "ae": "google.ae",
    "al": "google.al",
    "am": "google.am",
    "as": "google.as",
    "at": "google.at",
    "az": "google.az",
    "ba": "google.ba",
    "be": "google.be",
    "bf": "google.bf",
    "bg": "google.bg",
    "bi": "google.bi",
    "bj": "google.bj",
    "bs": "google.bs",
    "bt": "google.bt",
    "by": "google.by",
    "ca": "google.ca",
    "cg": "google.cg",
    "cf": "google.cf",
    "ch": "google.ch",
    "ci": "google.ci",
    "cl": "google.cl",
    "cm": "google.cm",
    "ao": "google.co.ao",
    "bw": "google.co.bw",
    "ck": "google.co.ck",
    "cr": "google.co.cr",
    "id": "google.co.id",
    "il": "google.co.il",
    "in": "google.co.in",
    "jp": "google.co.jp",
    "ke": "google.co.ke",
    "kr": "google.co.kr",
    "ls": "google.co.ls",
    "ma": "google.co.ma",
    "mz": "google.co.mz",
    "nz": "google.co.nz",
    "th": "google.co.th",
    "tz": "google.co.tz",
    "ug": "google.co.ug",
    "uk": "google.co.uk",
    "uz": "google.co.uz",
    "ve": "google.co.ve",
    "vi": "google.co.vi",
    "za": "google.co.za",
    "zm": "google.co.zm",
    "zw": "google.co.zw",
    "us": "google.com",
    "af": "google.com.af",
    "ag": "google.com.ag",
    "ai": "google.com.ai",
    "ar": "google.com.ar",
    "au": "google.com.au",
    "bd": "google.com.bd",
    "bh": "google.com.bh",
    "bn": "google.com.bn",
    "bo": "google.com.bo",
    "br": "google.com.br",
    "bz": "google.com.bz",
    "co": "google.com.co",
    "cu": "google.com.cu",
    "cy": "google.com.cy",
    "do": "google.com.do",
    "ec": "google.com.ec",
    "eg": "google.com.eg",
    "et": "google.com.et",
    "fj": "google.com.fj",
    "gh": "google.com.gh",
    "gi": "google.com.gi",
    "gt": "google.com.gt",
    "hk": "google.com.hk",
    "jm": "google.com.jm",
    "kh": "google.com.kh",
    "kw": "google.com.kw",
    "lb": "google.com.lb",
    "ly": "google.com.ly",
    "mm": "google.com.mm",
    "mt": "google.com.mt",
    "mx": "google.com.mx",
    "my": "google.com.my",
    "na": "google.com.na",
    "ng": "google.com.ng",
    "ni": "google.com.ni",
    "np": "google.com.np",
    "om": "google.com.om",
    "pa": "google.com.pa",
    "pe": "google.com.pe",
    "pg": "google.com.pg",
    "ph": "google.com.ph",
    "pk": "google.com.pk",
    "pr": "google.com.pr",
    "py": "google.com.py",
    "qa": "google.com.qa",
    "sa": "google.com.sa",
    "sb": "google.com.sb",
    "sg": "google.com.sg",
    "sl": "google.com.sl",
    "sv": "google.com.sv",
    "tj": "google.com.tj",
    "tr": "google.com.tr",
    "tw": "google.com.tw",
    "ua": "google.com.ua",
    "uy": "google.com.uy",
    "vc": "google.com.vc",
    "vn": "google.com.vn",
    "cv": "google.cv",
    "cz": "google.cz",
    "de": "google.de",
    "dj": "google.dj",
    "dk": "google.dk",
    "dm": "google.dm",
    "dz": "google.dz",
    "ee": "google.ee",
    "es": "google.es",
    "fi": "google.fi",
    "fm": "google.fm",
    "fr": "google.fr",
    "ga": "google.ga",
    "ge": "google.ge",
    "gl": "google.gl",
    "gm": "google.gm",
    "gp": "google.gp",
    "gr": "google.gr",
    "gy": "google.gy",
    "hn": "google.hn",
    "hr": "google.hr",
    "ht": "google.ht",
    "hu": "google.hu",
    "ie": "google.ie",
    "iq": "google.iq",
    "is": "google.is",
    "it": "google.it",
    "je": "google.je",
    "jo": "google.jo",
    "kg": "google.kg",
    "ki": "google.ki",
    "kz": "google.kz",
    "la": "google.la",
    "li": "google.li",
    "lk": "google.lk",
    "lt": "google.lt",
    "lu": "google.lu",
    "lv": "google.lv",
    "md": "google.md",
    "mg": "google.mg",
    "mk": "google.mk",
    "ml": "google.ml",
    "mn": "google.mn",
    "ms": "google.ms",
    "mu": "google.mu",
    "mv": "google.mv",
    "mw": "google.mw",
    "ne": "google.ne",
    "nl": "google.nl",
    "no": "google.no",
    "nr": "google.nr",
    "nu": "google.nu",
    "pl": "google.pl",
    "ps": "google.ps",
    "pt": "google.pt",
    "ro": "google.ro",
    "rs": "google.rs",
    "ru": "google.ru",
    "rw": "google.rw",
    "sc": "google.sc",
    "se": "google.se",
    "sh": "google.sh",
    "si": "google.si",
    "sk": "google.sk",
    "sm": "google.sm",
    "sn": "google.sn",
    "so": "google.so",
    "sr": "google.sr",
    "td": "google.td",
    "tg": "google.tg",
    "tk": "google.tk",
    "tl": "google.tl",
    "tm": "google.tm",
    "tn": "google.tn",
    "to": "google.to",
    "tt": "google.tt",
    "vg": "google.vg",
    "vu": "google.vu",
    "ws": "google.ws",
}
